function [u,x,t] = spec_full(X,T,N,f,nu,Dnu)
% Performs the spectral method for the equation, analytically
%       u_t + u_x  = (nu(x)u_x)_x, t > 0 and a < x < b
%       u(0,x) = f(x),  u(a,x) = u(t,b)
% MatLabs fft command is used initially, then the DFT and IDFT are computed
% by hand
%
%INPUT
%   X = Space interval [a,b].
%   T = Time interval [0,t].
%   N = Space and time partition numbers,[space,time], Best if space = 2^n.
%   f = Function handle for initial condition for t=0.
%   nu = Function handle for diffusion coefficient.
%   Dnu = Function handle for derivative of diffusion coefficient.
%
%OUTPUT
%   u = Approximation
%   x = Space decomposition
%   t = Time decomposition
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialization
a = X(1); b = X(2);
ta = T(1); tb = T(2);
n = N(1); m = N(2);

%% Time and space decompositions
h = (tb-ta)/m;
t = linspace(ta,tb,m+1);
x = linspace(a,b,n+1);
x(n+1) = [];

%% Compute FFT and arrange coefficients
fk = fft(f(x),n);
fk = fk([n/2+1:n 1:n/2]);
k = (-n/2:n/2-1);

%% Initialize solution
u = zeros(n,m-1);
u(:,1) = f(x);

%% Compute using full spectral approximation.
for J = 1:length(t)-1% time step
    L = 0;
    for K = 1:n
        L = L + 1;
        beta = 1i.*(Dnu(x(L)) - 1).*k - k.^2.*nu(x(L));
        uk = fk.*exp(beta.*(ta + J*h));
        u(L,J+1) = sum(uk.*exp(1i*k*x(L)))/n;
    end
end

%% Fix endpoint
u = real(u);
u(n+1,:) = u(1,:);
x = [x b];
