function [u,x,t] = spec_psuedo(X,T,N,f,nu,Dnu)
%Psuedo Spectral method, using the differtiaion matrix  with BDF 2 
% iterations for the equation:
%       u_t + u_x  = (nu(x)u_x)_x, t > 0 and a < x < b
%       u(0,x) = f(x),  u(a,x) = u(t,b)
%
%INPUT
%   X = Space interval [a,b].
%   T = Time interval [0,t].
%   N = Space and time partition numbers,[space,time], Best if space = 2^n.
%   f = Function handle for initial condition for t=0.
%   nu = Function handle for diffusion coefficient.
%   Dnu = Function handle for derivative of diffusion coefficient.
%         (only used to compute first step for BDF2 iteration)
%
%OUTPUT
%   u = Approximation
%   x = Space decomposition
%   t = Time decomposition
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Debug
dBug = 0;
if dBug
    close all; clear all; clc;
    dBug = 1;
    X = [0,2*pi];
    T = [0,1];
    nx = 2^5;
    N = [nx,20];    
    nu = @(x) .2 + 0*x;
    Dnu = @(x) 0 + 0*x;
    f = @(x) 3./(5-4*cos(x));
end

%% Initialization
a = X(1); b = X(2);
ta = T(1); tb = T(2);
n = N(1); m = N(2);

%% Time and space decompositions
t = linspace(ta,tb,m+1);
x = linspace(a,b,n+1);
x(n+1) = [];
dx = x(2) - x(1);
dt = t(2) - t(1);

%% Preallocation and inital condition
u = zeros(n,m);
u(:,1) = f(x);

%% BDF1 for first time step
% jVec = 2*pi/(b-a)*[0:n/2-1, 0, -n/2+1:-1];
% jVec2 = 2*pi/(b-a)*[0:n/2-1, n/2, -n/2+1:-1];
% fk = fft(u(:,1),n)';
% F = (1 - 1i*jVec - jVec2.^2);
% u(:,2) = ifft((1 - 2*dt*F/3).*fk,n);
fk = fft(u(:,1),n)';% Exact solution for first step.
fk = fk([n/2+1:n 1:n/2]);
k = (-n/2:n/2-1);
for K = 1:n
    beta = 1i.*(Dnu(x(K)) - 1).*k - k.^2.*nu(x(K));
    uk = fk.*exp(beta.*(ta + dt));
    u(K,2) = sum(uk.*exp(1i*k*x(K)))/n;
end

%% BDF2 Iterations
vec = [0 .5*(-1).^(1:n-1).*cot((1:n-1)*dx/2)]';
D = toeplitz(vec,vec([1 n:-1:2]));% Differentiation matrix
D(1:n+1:end) = zeros(n,1);
D(1:n+1:end) = sum(D,2);

I = speye(n,n);
nuM = sparse(diag(nu(x)));
for K = 2:m %BDF2
    u(:,K+1) = (I - 2*dt/3*(D*(nuM*D) - D))\(4/3*u(:,K)-u(:,K-1)/3);
end

%% Fix endpoint
u = real(u);
u(n+1,:) = u(1,:);
x = [x b];

if dBug
    sol = solution(x,t,nu); %#ok<*UNRCH>
    err = zeros(1,m);
    for K = 2:m+1
        err(K-1) = max(u(:,K-1)-sol(:,K-1));
    end

    figure(1)
    surf(t,x,u);
    title('approximation');

    figure(2)
    surf(t,x,sol);
    title('solution');

    figure(3)
    semilogy(t(2:end),err)
end

